@@ -16,7 +16,7 @@ from __future__ import annotations
 
 import json
 from abc import ABC, abstractmethod
-from typing import Any, Generic, TypeVar
+from typing import Any, Generic, TypeVar, cast
 
 from streamlit.runtime.secrets import AttrDict, secrets_singleton
 from streamlit.util import calc_md5
@@ -116,7 +116,9 @@ class BaseConnection(ABC, Generic[RawConnectionT]):
         if connections_section is None or type(connections_section) is not AttrDict:
             return AttrDict({})
 
-        return connections_section.get(self._connection_name, AttrDict({}))
+        return cast(
+            "AttrDict", connections_section.get(self._connection_name, AttrDict({}))
+        )
 
     def reset(self) -> None:
         """Reset this connection so that it gets reinitialized the next time it's used.
