@@ -64,7 +64,9 @@ from streamlit.elements.layouts import LayoutsMixin
 from streamlit.elements.lib.form_utils import FormData, current_form_id
 from streamlit.elements.lib.layout_utils import (
     get_height_config,
+    get_text_alignment_config,
     get_width_config,
+    validate_text_alignment,
 )
 from streamlit.elements.map import MapMixin
 from streamlit.elements.markdown import MarkdownMixin
@@ -495,6 +497,11 @@ class DeltaGenerator(
                 msg.delta.new_element.width_config.CopyFrom(
                     get_width_config(layout_config.width)
                 )
+            if layout_config.text_alignment is not None:
+                validate_text_alignment(layout_config.text_alignment)
+                msg.delta.new_element.text_alignment_config.CopyFrom(
+                    get_text_alignment_config(layout_config.text_alignment)
+                )
 
         # Only enqueue message and fill in metadata if there's a container.
         msg_was_enqueued = False
