@@ -201,9 +201,15 @@ def _parse_value(
     import pandas as pd
 
     try:
-        if column_data_kind in (ColumnDataKind.LIST, ColumnDataKind.EMPTY):
+        if column_data_kind == ColumnDataKind.LIST:
             return list(value) if is_list_like(value) else [value]  # ty: ignore
 
+        if column_data_kind == ColumnDataKind.EMPTY:
+            # For empty columns, preserve the value type from the frontend.
+            # If it's a list (e.g., from multiselect), return as list.
+            # If it's a scalar (e.g., from number input), return as scalar.
+            return list(value) if is_list_like(value) else value  # ty: ignore
+
         if column_data_kind == ColumnDataKind.STRING:
             return str(value)
 
