@@ -529,7 +529,9 @@ class ScriptRunner:
                     widget_ids = {w.id for w in rerun_data.widget_states.widgets}
                 self._session_state.on_script_finished(widget_ids)
 
-            fragment_ids_this_run = list(rerun_data.fragment_id_queue)
+            fragment_ids_this_run: list[str] | None = (
+                rerun_data.fragment_id_queue or None
+            )
 
             ctx.reset(
                 query_string=rerun_data.query_string,
